//#################################################
//# Code for the MS8892 Evalboard Demo            #
//#                                               #
//# Versions:                                     #
//#  V2/28.01.22                                  #
//#                                               #
//#################################################

#define VERSION_TXT "MS8892_Evalboard_Demo v1"

#include <Wire.h>

#define MS8892_ADDR 0x24    // Address when pin OPOL_A0 connected to GND
// #define MS8892_ADDR 0x25    // Address when pin OPOL_A0 connected to VDD

#define MCS    0x00
#define RCS    0x01
#define COMP   0x02
#define RRES   0x03
#define LCLR   0x04
#define WTH    0x05
#define RTH    0x06
#define WOPT1  0x07
#define WOPT2  0x09

bool TH_err = false;

void setup() {
  pinMode(LED_BUILTIN, OUTPUT);
  Wire.begin();

  // set RAM Mode, invert polarity, rel. threshold
  write_data(MS8892_ADDR, WOPT2, 0b01001001);
  // set relative threshold step -10 
  write_data(MS8892_ADDR, WTH, 10);
  // measure capacitance as threshold baseline
  write_command(MS8892_ADDR, MCS); delay(2);
  // check the threshold error flag
  if (read_data(MS8892_ADDR, RRES) & 0b00001000)
    TH_err = true;
  // set periodic comparison mode with 8 measurements/s
  write_data(MS8892_ADDR, WOPT1, 0b00001000);
}


void loop() {
  // Arduino onboard LED flashes if threshold over/underflow
  digitalWrite(LED_BUILTIN, HIGH);
  delay(100);
  digitalWrite(LED_BUILTIN, LOW);

  // 2s heartbeat if threshold ok
  // fast flashing if threshold error occurred
  if (TH_err)
    delay(100);
  else
    delay(1900);
}


/*******************************************************
 *  I2C access routines
 *******************************************************/

void write_data(int address, int command, int databyte)
{
  Wire.beginTransmission(address);
  Wire.write(command);
  Wire.write(databyte);
  Wire.endTransmission(); 
}

int  read_data(int address, int command)
{
  int    value = 0;
  
  Wire.beginTransmission(address);
  Wire.write(command);
  Wire.endTransmission(false);
  Wire.requestFrom(address,1);
  value = Wire.read();
  return (value);
}

void write_command(int address, int command)
{
  Wire.beginTransmission(address);
  Wire.write(command);
  Wire.endTransmission(true); 
}
